@@ -5,15 +5,15 @@
  * Author:
  *   Nat Friedman (nat@ximian.com)
  *
- * Copyright 1999, 2000, Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  */
 
 #include <config.h>
 
 #include "e-book-view-listener.h"
 #include "e-book-view.h"
+#include "e-book-view-private.h"
 #include "e-book.h"
-#include "e-book-marshal.h"
 
 static GObjectClass *parent_class;
 
@@ -21,7 +21,7 @@ struct _EBookViewPrivate {
 	GNOME_Evolution_Addressbook_BookView     corba_book_view;
 
 	EBook                 *book;
-	
+
 	EBookViewListener     *listener;
 
 	int                    response_id;
@@ -149,7 +149,9 @@ e_book_view_construct (EBookView *book_view, GNOME_Evolution_Addressbook_BookVie
  * @corba_book_view: a CORBA BookView object
  * @listener: an #EBookViewListener
  *
- * Creates a new #EBookView based on @corba_book_view and listening to @listener.
+ * Creates a new #EBookView based on @corba_book_view and listening to
+ * @listener.  This is a private function, applications should call
+ * #e_book_get_book_view or #e_book_async_get_book_view.
  *
  * Return value: A new #EBookView.
  **/
@@ -202,12 +204,13 @@ e_book_view_get_book (EBookView *book_view)
 
 	return book_view->priv->book;
 }
+
 /**
  * e_book_view_start:
  * @book_view: an #EBookView
  *
  * Tells @book_view to start processing events.
- **/
+ */
 void
 e_book_view_start (EBookView *book_view)
 {
@@ -313,7 +316,7 @@ e_book_view_class_init (EBookViewClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (EBookViewClass, contacts_changed),
 			      NULL, NULL,
-			      e_book_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 
@@ -323,7 +326,7 @@ e_book_view_class_init (EBookViewClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (EBookViewClass, contacts_added),
 			      NULL, NULL,
-			      e_book_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 
@@ -333,7 +336,7 @@ e_book_view_class_init (EBookViewClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (EBookViewClass, contacts_removed),
 			      NULL, NULL,
-			      e_book_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 
@@ -343,7 +346,7 @@ e_book_view_class_init (EBookViewClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (EBookViewClass, sequence_complete),
 			      NULL, NULL,
-			      e_book_marshal_NONE__INT,
+			      g_cclosure_marshal_VOID__INT,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_INT);
 
@@ -353,7 +356,7 @@ e_book_view_class_init (EBookViewClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (EBookViewClass, status_message),
 			      NULL, NULL,
-			      e_book_marshal_NONE__STRING,
+			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
 
