@@ -198,8 +198,10 @@ camel_folder_finalize (CamelObject *object)
 	if (camel_folder->parent_store)
 		camel_object_unref (camel_folder->parent_store);
 
-	if (camel_folder->summary)
+	if (camel_folder->summary) {
+		camel_folder->summary->folder = NULL;
 		camel_object_unref (camel_folder->summary);
+	}
 
 	camel_folder_change_info_free(p->changed_frozen);
 
@@ -265,19 +267,33 @@ camel_folder_construct (CamelFolder *folder, CamelStore *parent_store,
 	store_db_path = g_build_filename (service->url->path, CAMEL_DB_FILE, NULL);
 	camel_exception_init(&ex);
 	if (strlen (store_db_path) < 2) {
+		char *store_path = camel_session_get_storage_path ((CamelSession *)camel_service_get_session (service), service, &ex);
+
 		g_free (store_db_path);
-		store_db_path = g_build_filename ( camel_session_get_storage_path ((CamelSession *)camel_service_get_session (service), service, &ex), CAMEL_DB_FILE, NULL);		
+		store_db_path = g_build_filename (store_path, CAMEL_DB_FILE, NULL);
+		g_free (store_path);
 	}
 
-
 	folder->cdb = camel_db_open (store_db_path, &ex);
-	g_free (store_db_path);
-	
 	if (camel_exception_is_set (&ex)) {
-		g_print ("Exiting without success for stire_db_path : [%s]: %s\n", store_db_path, camel_exception_get_description(&ex));
+		char *store_path;
+		
+		g_print ("Failure for store_db_path : [%s]\n", store_db_path);
+		g_free (store_db_path);		
+
+		store_path =   camel_session_get_storage_path ((CamelSession *)camel_service_get_session (service), service, &ex);
+		store_db_path = g_build_filename (store_path, CAMEL_DB_FILE, NULL);
+		g_free (store_path);
 		camel_exception_clear(&ex);
-		return;
-	}	
+		folder->cdb = camel_db_open (store_db_path, &ex);
+		if (camel_exception_is_set (&ex)) {
+			g_print("Retry with %s failed\n", store_db_path);
+			g_free(store_db_path);
+			camel_exception_clear(&ex);
+			return;
+		}
+	}
+	g_free (store_db_path);
 }
 
 
@@ -331,11 +347,7 @@ camel_folder_refresh_info (CamelFolder *folder, CamelException *ex)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 
-	CAMEL_FOLDER_REC_LOCK(folder, lock);
-
 	CF_CLASS (folder)->refresh_info (folder, ex);
-	
-	CAMEL_FOLDER_REC_UNLOCK(folder, lock);
 }
 
 static int
@@ -384,23 +396,13 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 
 			#warning "Add a better base class function to get counts specific to normal/vee folder."
 			if (unread == -1) {
-				int j;
-				CamelMessageInfo *info;
 
-				if (!CAMEL_IS_VEE_FOLDER (folder)) {
-					/* TODO: Locking? */
+				if (1) {
 					unread = folder->summary->unread_count;
 					deleted = folder->summary->deleted_count;
 					junked = folder->summary->junk_count;
 					junked_not_deleted = folder->summary->junk_not_deleted_count;
 					visible = folder->summary->visible_count;
-                                        #warning "unread should be unread and not del/junk and take care of dirty infos also"
-					// camel_folder_summary_save_to_db (folder->summary, NULL);
-					//camel_db_count_visible_unread_message_info (folder->cdb, folder->full_name, &unread, ex);
-					//camel_db_count_junk_message_info (folder->cdb, folder->full_name, &junked, ex);
-					//camel_db_count_deleted_message_info (folder->cdb, folder->full_name, &deleted, ex);
-					//camel_db_count_junk_not_deleted_message_info (folder->cdb, folder->full_name, &junked_not_deleted, ex);
-					//camel_db_count_visible_message_info (folder->cdb, folder->full_name, &visible, ex);
 				} else {
 					/* count = camel_folder_summary_count (folder->summary);
 					for (j = 0; j < count; j++) {
@@ -528,7 +530,7 @@ get_name (CamelFolder *folder)
  * Get the (short) name of the folder. The fully qualified name
  * can be obtained with the #camel_folder_get_full_name method.
  *
- * Returns the short name of the folder
+ * Returns: the short name of the folder
  **/
 const char *
 camel_folder_get_name (CamelFolder *folder)
@@ -552,7 +554,7 @@ get_full_name (CamelFolder *folder)
  *
  * Get the full name of the folder.
  *
- * Returns the full name of the folder
+ * Returns: the full name of the folder
  **/
 const char *
 camel_folder_get_full_name (CamelFolder *folder)
@@ -574,7 +576,7 @@ get_parent_store (CamelFolder * folder)
  * camel_folder_get_parent_store:
  * @folder: a #CamelFolder object
  *
- * Returns the parent #CamelStore of the folder
+ * Returns: the parent #CamelStore of the folder
  **/
 CamelStore *
 camel_folder_get_parent_store (CamelFolder *folder)
@@ -626,7 +628,7 @@ get_message_count (CamelFolder *folder)
  * camel_folder_get_message_count:
  * @folder: a #CamelFolder object
  *
- * Returns the number of messages in the folder, or %-1 if unknown
+ * Returns: the number of messages in the folder, or %-1 if unknown
  **/
 int
 camel_folder_get_message_count (CamelFolder *folder)
@@ -647,7 +649,7 @@ camel_folder_get_message_count (CamelFolder *folder)
  *
  * DEPRECATED: use #camel_object_get instead.
  *
- * Returns the number of unread messages in the folder, or %-1 if
+ * Returns: the number of unread messages in the folder, or %-1 if
  * unknown
  **/
 int
@@ -667,7 +669,7 @@ camel_folder_get_unread_message_count (CamelFolder *folder)
  * camel_folder_get_deleted_message_count:
  * @folder: a #CamelFolder object
  *
- * Returns the number of deleted messages in the folder, or %-1 if
+ * Returns: the number of deleted messages in the folder, or %-1 if
  * unknown
  **/
 int
@@ -698,7 +700,6 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 
 }
 
-
 /**
  * camel_folder_append_message:
  * @folder: a #CamelFolder object
@@ -720,9 +721,7 @@ camel_folder_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 
 	CAMEL_FOLDER_REC_LOCK(folder, lock);
-
 	CF_CLASS (folder)->append_message (folder, message, info, appended_uid, ex);
-
 	CAMEL_FOLDER_REC_UNLOCK(folder, lock);
 }
 
@@ -738,7 +737,7 @@ get_permanent_flags (CamelFolder *folder)
  * camel_folder_get_permanent_flags:
  * @folder: a #CamelFolder object
  *
- * Returns the set of #CamelMessageFlags that can be permanently
+ * Returns: the set of #CamelMessageFlags that can be permanently
  * stored on a message between sessions. If it includes
  * #CAMEL_FLAG_USER, then user-defined flags will be remembered.
  **/
@@ -776,7 +775,7 @@ get_message_flags(CamelFolder *folder, const char *uid)
  *
  * Deprecated: Use #camel_folder_get_message_info instead.
  *
- * Returns the #CamelMessageFlags that are set on the indicated
+ * Returns: the #CamelMessageFlags that are set on the indicated
  * message.
  **/
 guint32
@@ -827,7 +826,7 @@ set_message_flags(CamelFolder *folder, const char *uid, guint32 flags, guint32 s
  * DEPRECATED: Use #camel_message_info_set_flags on the message info directly
  * (when it works)
  *
- * Returns %TRUE if the flags were changed or %FALSE otherwise
+ * Returns: %TRUE if the flags were changed or %FALSE otherwise
  **/
 gboolean
 camel_folder_set_message_flags(CamelFolder *folder, const char *uid, guint32 flags, guint32 set)
@@ -870,7 +869,7 @@ get_message_user_flag(CamelFolder *folder, const char *uid, const char *name)
  * DEPRECATED: Use #camel_message_info_get_user_flag on the message
  * info directly
  *
- * Returns %TRUE if the given user flag is set on the message or
+ * Returns: %TRUE if the given user flag is set on the message or
  * %FALSE otherwise
  **/
 gboolean
@@ -953,7 +952,7 @@ get_message_user_tag(CamelFolder *folder, const char *uid, const char *name)
  * DEPRECATED: Use #camel_message_info_get_user_tag on the
  * #CamelMessageInfo directly.
  *
- * Returns the value of the user tag
+ * Returns: the value of the user tag
  **/
 const char *
 camel_folder_get_message_user_tag (CamelFolder *folder, const char *uid,  const char *name)
@@ -1023,7 +1022,7 @@ get_message_info (CamelFolder *folder, const char *uid)
  * Retrieve the #CamelMessageInfo for the specified @uid.  This return
  * must be freed using #camel_folder_free_message_info.
  *
- * Returns the summary information for the indicated message, or %NULL
+ * Returns: the summary information for the indicated message, or %NULL
  * if the uid does not exist
  **/
 CamelMessageInfo *
@@ -1101,7 +1100,7 @@ camel_folder_ref_message_info(CamelFolder *folder, CamelMessageInfo *info)
  *
  * Get whether or not the folder has a summary.
  *
- * Returns %TRUE if a summary is available or %FALSE otherwise
+ * Returns: %TRUE if a summary is available or %FALSE otherwise
  **/
 gboolean
 camel_folder_has_summary_capability (CamelFolder *folder)
@@ -1132,7 +1131,7 @@ get_message (CamelFolder *folder, const char *uid, CamelException *ex)
  *
  * Get a message from its UID in the folder.
  *
- * Returns a #CamelMimeMessage corresponding to @uid
+ * Returns: a #CamelMimeMessage corresponding to @uid
  **/
 CamelMimeMessage *
 camel_folder_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
@@ -1192,7 +1191,7 @@ get_uids(CamelFolder *folder)
  * support summaries. The returned array should not be modified, and
  * must be freed by passing it to #camel_folder_free_uids.
  *
- * Returns a GPtrArray of UIDs corresponding to the messages available
+ * Returns: a GPtrArray of UIDs corresponding to the messages available
  * in the folder
  **/
 GPtrArray *
@@ -1290,7 +1289,7 @@ get_summary(CamelFolder *folder)
  * should not be modified, and must be freed with
  * #camel_folder_free_summary.
  *
- * Returns an array of #CamelMessageInfo
+ * Returns: an array of #CamelMessageInfo
  **/
 GPtrArray *
 camel_folder_get_summary (CamelFolder *folder)
@@ -1332,7 +1331,7 @@ camel_folder_free_summary(CamelFolder *folder, GPtrArray *array)
  *
  * Checks if a folder supports searching.
  *
- * Returns %TRUE if the folder supports searching or %FALSE otherwise
+ * Returns: %TRUE if the folder supports searching or %FALSE otherwise
  **/
 gboolean
 camel_folder_has_search_capability (CamelFolder *folder)
@@ -1365,7 +1364,7 @@ search_by_expression (CamelFolder *folder, const char *expression,
  *
  * Searches the folder for messages matching the given search expression.
  *
- * Returns a #GPtrArray of uids of matching messages. The caller must
+ * Returns: a #GPtrArray of uids of matching messages. The caller must
  * free the list and each of the elements when it is done.
  **/
 GPtrArray *
@@ -1407,7 +1406,7 @@ search_by_uids(CamelFolder *folder, const char *exp, GPtrArray *uids, CamelExcep
  *
  * Search a subset of uid's for an expression match.
  *
- * Returns a #GPtrArray of uids of matching messages. The caller must
+ * Returns: a #GPtrArray of uids of matching messages. The caller must
  * free the list and each of the elements when it is done.
  **/
 GPtrArray *
@@ -1596,9 +1595,12 @@ camel_folder_delete (CamelFolder *folder)
 	folder->folder_flags |= CAMEL_FOLDER_HAS_BEEN_DELETED;
 	
 	CF_CLASS (folder)->delete (folder);
-
+	
 	CAMEL_FOLDER_REC_UNLOCK (folder, lock);
 
+	/* Delete the references of the folder from the DB.*/
+	camel_db_delete_folder (folder->cdb, folder->full_name, NULL);
+	
 	camel_object_trigger_event (folder, "deleted", NULL);
 }
 
@@ -1635,7 +1637,7 @@ camel_folder_rename(CamelFolder *folder, const char *new)
 	old = g_strdup(folder->full_name);
 
 	CF_CLASS (folder)->rename(folder, new);
-
+	camel_db_rename_folder (folder->cdb, old, new, NULL);
 	camel_object_trigger_event (folder, "renamed", old);
 	g_free(old);
 }
@@ -1725,7 +1727,7 @@ is_frozen (CamelFolder *folder)
  * camel_folder_is_frozen:
  * @folder: a #CamelFolder object
  *
- * Returns whether or not the folder is frozen
+ * Returns: whether or not the folder is frozen
  **/
 gboolean
 camel_folder_is_frozen (CamelFolder *folder)
@@ -1745,7 +1747,7 @@ get_quota_info (CamelFolder *folder)
  * camel_folder_get_quota_info:
  * @folder: a #CamelFolder object
  *
- * Returns list of known quota(s) for the folder.
+ * Returns: list of known quota(s) for the folder.
  **/
 CamelFolderQuotaInfo *
 camel_folder_get_quota_info (CamelFolder *folder)
@@ -1761,8 +1763,8 @@ camel_folder_get_quota_info (CamelFolder *folder)
  * @used: Current usage of the quota.
  * @total: Total available size of the quota.
  *
- * Returns newly allocated #CamelFolderQuotaInfo structure with initialized values
- * based on the parameters, with next member set to NULL.
+ * Returns: newly allocated #CamelFolderQuotaInfo structure with
+ * initialized values based on the parameters, with next member set to NULL.
  **/
 CamelFolderQuotaInfo *
 camel_folder_quota_info_new (const char *name, guint64 used, guint64 total)
@@ -1940,6 +1942,7 @@ filter_free(CamelSession *session, CamelSessionThreadMsg *msg)
 	if (m->notjunk)
 		camel_folder_free_deep(m->folder, m->notjunk);
 
+	camel_folder_summary_save_to_db (m->folder->summary, &m->ex);
 	camel_folder_thaw(m->folder);
 	camel_object_unref(m->folder);
 }
@@ -2116,7 +2119,7 @@ camel_folder_free_deep (CamelFolder *folder, GPtrArray *array)
  * Change info structures are not MT-SAFE and must be
  * locked for exclusive access externally.
  *
- * Returns a new #CamelFolderChangeInfo
+ * Returns: a new #CamelFolderChangeInfo
  **/
 CamelFolderChangeInfo *
 camel_folder_change_info_new(void)
@@ -2473,7 +2476,7 @@ camel_folder_change_info_recent_uid(CamelFolderChangeInfo *info, const char *uid
  *
  * Gets whether or not there have been any changes.
  *
- * Returns %TRUE if the changeset contains any changes or %FALSE
+ * Returns: %TRUE if the changeset contains any changes or %FALSE
  * otherwise
  **/
 gboolean
