@@ -1,6 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  Camel
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
+ *
+ *  Authors: Jeffrey Stedfast <fejj@novell.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -14,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -22,28 +24,29 @@
 #include <config.h>
 #endif
 
-#include <glib.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <dirent.h>
 #include <unistd.h>
+#include <dirent.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <time.h>
 
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+
 #include <libedataserver/e-data-server-util.h>
-#include <camel/camel-utf8.h>
+
 #include <camel/camel-private.h>
 #include <camel/camel-file-utils.h>
-#include <camel/camel-mime-message.h>
-#include <camel/camel-stream-mem.h>
 #include <camel/camel-stream-filter.h>
 #include <camel/camel-mime-filter-crlf.h>
-#include <camel/camel-i18n.h>
+#include <camel/camel-mime-message.h>
+#include <camel/camel-stream-mem.h>
+#include <camel/camel-utf8.h>
 
 #include "camel-imap4-utils.h"
 #include "camel-imap4-store.h"
@@ -83,7 +86,9 @@ static CamelOfflineFolderClass *parent_class = NULL;
 static GSList *imap4_folder_props = NULL;
 
 static CamelProperty imap4_prop_list[] = {
-	{ CAMEL_IMAP4_FOLDER_ENABLE_MLIST, "mlist_info", N_("Enable Mailing-List detection required for some filter and vFolder rules") },
+	{ CAMEL_IMAP4_FOLDER_ENABLE_MLIST, "mlist_info", N_("Enable extended Mailing-List detection required for some filter and vFolder rules") },
+	{ CAMEL_IMAP4_FOLDER_EXPIRE_ACCESS, "expire-access", N_("Expire cached messages that haven't been read in X seconds") },
+	{ CAMEL_IMAP4_FOLDER_EXPIRE_AGE, "expire-age", N_("Expire cached messages older than X seconds") },
 };
 
 
@@ -154,8 +159,7 @@ camel_imap4_folder_finalize (CamelObject *object)
 	
 	camel_object_unref (folder->search);
 	
-	if (folder->cache)
-		camel_object_unref (folder->cache);
+	camel_object_unref (folder->cache);
 	
 	if (folder->journal) {
 		camel_offline_journal_write (folder->journal, NULL);
@@ -169,6 +173,7 @@ camel_imap4_folder_finalize (CamelObject *object)
 static int
 imap4_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
 {
+	CamelIMAP4Folder *folder = (CamelIMAP4Folder *) object;
 	CamelArgGetV props;
 	int i, count = 0;
 	guint32 tag;
@@ -187,7 +192,13 @@ imap4_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
 			*arg->ca_ptr = g_slist_concat (*arg->ca_ptr, g_slist_copy (imap4_folder_props));
 			break;
 		case CAMEL_IMAP4_FOLDER_ARG_ENABLE_MLIST:
-			*arg->ca_int = ((CamelIMAP4Folder *) object)->enable_mlist;
+			*arg->ca_int = folder->enable_mlist;
+			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_ACCESS:
+			*arg->ca_int = folder->cache->expire_access;
+			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_AGE:
+			*arg->ca_int = folder->cache->expire_age;
 			break;
 		default:
 			count++;
@@ -207,6 +218,7 @@ static int
 imap4_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
 {
 	CamelIMAP4Folder *folder = (CamelIMAP4Folder *) object;
+	CamelDataCache *cache = folder->cache;
 	gboolean save = FALSE;
 	guint32 tag;
 	int i;
@@ -223,6 +235,18 @@ imap4_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
 				save = TRUE;
 			}
 			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_ACCESS:
+			if (cache->expire_access != (time_t) arg->ca_int) {
+				camel_data_cache_set_expire_access (cache, (time_t) arg->ca_int);
+				save = TRUE;
+			}
+			break;
+		case CAMEL_IMAP4_FOLDER_ARG_EXPIRE_AGE:
+			if (cache->expire_age != (time_t) arg->ca_int) {
+				camel_data_cache_set_expire_age (cache, (time_t) arg->ca_int);
+				save = TRUE;
+			}
+			break;
 		default:
 			continue;
 		}
@@ -364,7 +388,7 @@ camel_imap4_folder_new (CamelStore *store, const char *full_name, CamelException
 	
 	if (camel_object_state_read (folder) == -1) {
 		/* set our defaults */
-		imap4_folder->enable_mlist = TRUE;
+		imap4_folder->enable_mlist = FALSE;
 	}
 	
 	if (!g_ascii_strcasecmp (full_name, "INBOX")) {
@@ -396,7 +420,7 @@ camel_imap4_folder_new (CamelStore *store, const char *full_name, CamelException
 		/* we *do* care if summary loading fails here though */
 		if (camel_folder_summary_load (folder->summary) == -1) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID_PATH,
-					      _("Cannot access folder `%s': %s"),
+					      _("Cannot access folder '%s': %s"),
 					      full_name, g_strerror (ENOENT));
 			
 			camel_object_unref (folder);
@@ -455,13 +479,13 @@ imap4_sync_flag (CamelFolder *folder, GPtrArray *infos, char onoff, const char *
 		case CAMEL_IMAP4_RESULT_NO:
 			/* FIXME: would be good to save the NO reason into the err message */
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot sync flags to folder `%s': Unknown"),
+					      _("Cannot sync flags to folder '%s': Unknown error"),
 					      folder->full_name);
 			retval = -1;
 			break;
 		case CAMEL_IMAP4_RESULT_BAD:
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot sync flags to folder `%s': Bad command"),
+					      _("Cannot sync flags to folder '%s': Bad command"),
 					      folder->full_name);
 			retval = -1;
 			break;
@@ -601,12 +625,12 @@ imap4_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		case CAMEL_IMAP4_RESULT_NO:
 			/* FIXME: would be good to save the NO reason into the err message */
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot expunge folder `%s': Unknown"),
+					      _("Cannot expunge folder '%s': Unknown error"),
 					      folder->full_name);
 			break;
 		case CAMEL_IMAP4_RESULT_BAD:
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot expunge folder `%s': Bad command"),
+					      _("Cannot expunge folder '%s': Bad command"),
 					      folder->full_name);
 			break;
 		}
@@ -692,7 +716,7 @@ untagged_fetch (CamelIMAP4Engine *engine, CamelIMAP4Command *ic, guint32 index,
 	do {
 		if (camel_imap4_engine_next_token (engine, token, ex) == -1)
 			goto exception;
-		
+
 		if (token->token == ')' || token->token == '\n')
 			break;
 		
@@ -862,12 +886,12 @@ imap4_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: would be good to save the NO reason into the err message */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot get message %s from folder `%s': No such message"),
+				      _("Cannot get message %s from folder '%s': No such message"),
 				      uid, folder->full_name);
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot get message %s from folder `%s': Bad command"),
+				      _("Cannot get message %s from folder '%s': Bad command"),
 				      uid, folder->full_name);
 		break;
 	}
@@ -911,7 +935,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	
 	if (((CamelIMAP4Folder *) folder)->read_only) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot append message to folder `%s': Folder is read-only"),
+				      _("Cannot append message to folder '%s': Folder is read-only"),
 				      folder->full_name);
 		return;
 	}
@@ -996,7 +1020,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 			resp = ic->resp_codes->pdata[i];
 			if (resp->code == CAMEL_IMAP4_RESP_CODE_APPENDUID) {
 				if (resp->v.appenduid.uidvalidity == summary->uidvalidity)
-					*appended_uid = g_strdup_printf ("%lu", resp->v.appenduid.uid);
+					*appended_uid = g_strdup_printf ("%u", resp->v.appenduid.uid);
 				break;
 			}
 		}
@@ -1004,7 +1028,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 	case CAMEL_IMAP4_RESULT_NO:
 		/* FIXME: can we give the user any more information? */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot append message to folder `%s': Unknown error"),
+				      _("Cannot append message to folder '%s': Unknown error"),
 				      folder->full_name);
 		
 		for (i = 0; i < ic->resp_codes->len; i++) {
@@ -1034,7 +1058,7 @@ imap4_append_message (CamelFolder *folder, CamelMimeMessage *message,
 		break;
 	case CAMEL_IMAP4_RESULT_BAD:
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Cannot append message to folder `%s': Bad command"),
+				      _("Cannot append message to folder '%s': Bad command"),
 				      folder->full_name);
 		
 		break;
@@ -1154,11 +1178,11 @@ imap4_transfer_messages_to (CamelFolder *src, GPtrArray *uids, CamelFolder *dest
 			/* FIXME: would be good to save the NO reason into the err message */
 			if (move) {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot move messages from folder `%s' to folder `%s': Unknown"),
+						      _("Cannot move messages from folder '%s' to folder '%s': Unknown error"),
 						      src->full_name, dest->full_name);
 			} else {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot copy messages from folder `%s' to folder `%s': Unknown"),
+						      _("Cannot copy messages from folder '%s' to folder '%s': Unknown error"),
 						      src->full_name, dest->full_name);
 			}
 			
@@ -1166,11 +1190,11 @@ imap4_transfer_messages_to (CamelFolder *src, GPtrArray *uids, CamelFolder *dest
 		case CAMEL_IMAP4_RESULT_BAD:
 			if (move) {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot move messages from folder `%s' to folder `%s': Bad command"),
+						      _("Cannot move messages from folder '%s' to folder '%s': Bad command"),
 						      src->full_name, dest->full_name);
 			} else {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot copy messages from folder `%s' to folder `%s': Bad command"),
+						      _("Cannot copy messages from folder '%s' to folder '%s': Bad command"),
 						      src->full_name, dest->full_name);
 			}
 			
