@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-source-list.c
  *
- * Copyright (C) 2003  Ximian, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Ettore Perazzoli <ettore@ximian.com>
  */
@@ -25,11 +25,8 @@
 #endif
 
 #include <string.h>
-#include "e-data-server-marshal.h"
 #include "e-source-list.h"
 
-static GObjectClass *parent_class = NULL;
-
 struct _ESourceListPrivate {
 	GConfClient *gconf_client;
 	char *gconf_path;
@@ -84,12 +81,12 @@ load_from_gconf (ESourceList *list)
 	new_groups_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	for (p = conf_list, pos = 0; p != NULL; p = p->next, pos++) {
-		const char *xml = p->data;
+		const xmlChar *xml = p->data;
 		xmlDocPtr xmldoc;
 		char *group_uid;
 		ESourceGroup *existing_group;
 
-		xmldoc = xmlParseDoc ((char *) xml);
+		xmldoc = xmlParseDoc (xml);
 		if (xmldoc == NULL)
 			continue;
 
@@ -231,6 +228,8 @@ conf_changed_callback (GConfClient *client,
 
 /* GObject methods.  */
 
+G_DEFINE_TYPE (ESourceList, e_source_list, G_TYPE_OBJECT);
+
 static void
 impl_dispose (GObject *object)
 {
@@ -241,7 +240,7 @@ impl_dispose (GObject *object)
 
 		g_source_remove (priv->sync_idle_id);
 		priv->sync_idle_id = 0;
-		
+
 		if (! e_source_list_sync (E_SOURCE_LIST (object), &error))
 			g_warning ("Could not update \"%s\": %s",
 				   priv->gconf_path, error->message);
@@ -266,9 +265,11 @@ impl_dispose (GObject *object)
 
 		g_object_unref (priv->gconf_client);
 		priv->gconf_client = NULL;
+	} else {
+		g_assert_not_reached ();
 	}
 
-	(* G_OBJECT_CLASS (parent_class)->dispose) (object);
+	(* G_OBJECT_CLASS (e_source_list_parent_class)->dispose) (object);
 }
 
 static void
@@ -285,7 +286,7 @@ impl_finalize (GObject *object)
 	g_free (priv->gconf_path);
 	g_free (priv);
 
-	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	(* G_OBJECT_CLASS (e_source_list_parent_class)->finalize) (object);
 }
 
 
@@ -299,34 +300,32 @@ e_source_list_class_init (ESourceListClass *class)
 	object_class->dispose  = impl_dispose;
 	object_class->finalize = impl_finalize;
 
-	parent_class = g_type_class_peek_parent (class);
-
-	signals[CHANGED] = 
+	signals[CHANGED] =
 		g_signal_new ("changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ESourceListClass, changed),
 			      NULL, NULL,
-			      e_data_server_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-	signals[GROUP_REMOVED] = 
+	signals[GROUP_REMOVED] =
 		g_signal_new ("group_removed",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ESourceListClass, group_removed),
 			      NULL, NULL,
-			      e_data_server_marshal_VOID__OBJECT,
+			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 
-	signals[GROUP_ADDED] = 
+	signals[GROUP_ADDED] =
 		g_signal_new ("group_added",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ESourceListClass, group_added),
 			      NULL, NULL,
-			      e_data_server_marshal_VOID__OBJECT,
+			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 }
@@ -341,26 +340,18 @@ e_source_list_init (ESourceList *source_list)
 	source_list->priv = priv;
 }
 
-GType
-e_source_list_get_type (void)
+/* returns the type */
+static GType
+get_source_list_type (void)
 {
-	static GType e_source_list_type = 0;
+	static GType type = 0;
 
-	if (!e_source_list_type) {
-		static GTypeInfo info = {
-                        sizeof (ESourceListClass),
-                        (GBaseInitFunc) NULL,
-                        (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) e_source_list_class_init,
-                        NULL, NULL,
-                        sizeof (ESourceList),
-                        0,
-                        (GInstanceInitFunc) e_source_list_init
-                };
-		e_source_list_type = g_type_register_static (G_TYPE_OBJECT, "ESourceList", &info, 0);
+	if (!type) {
+		if (!(type = g_type_from_name ("ESourceList")))
+			type = e_source_list_get_type ();
 	}
 
-	return e_source_list_type;
+	return type;
 }
 
 /* Public methods.  */
@@ -368,7 +359,7 @@ e_source_list_get_type (void)
 ESourceList *
 e_source_list_new (void)
 {
-	ESourceList *list = g_object_new (e_source_list_get_type (), NULL);
+	ESourceList *list = g_object_new (get_source_list_type (), NULL);
 
 	return list;
 }
@@ -382,7 +373,7 @@ e_source_list_new_for_gconf (GConfClient *client,
 	g_return_val_if_fail (GCONF_IS_CLIENT (client), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 
-	list = g_object_new (e_source_list_get_type (), NULL);
+	list = g_object_new (get_source_list_type (), NULL);
 
 	list->priv->gconf_path = g_strdup (path);
 	list->priv->gconf_client = client;
@@ -401,12 +392,12 @@ e_source_list_new_for_gconf (GConfClient *client,
 
 ESourceList *
 e_source_list_new_for_gconf_default (const char  *path)
-{	
+{
 	ESourceList *list;
 
 	g_return_val_if_fail (path != NULL, NULL);
 
-	list = g_object_new (e_source_list_get_type (), NULL);
+	list = g_object_new (get_source_list_type (), NULL);
 
 	list->priv->gconf_path = g_strdup (path);
 	list->priv->gconf_client = gconf_client_get_default ();
@@ -480,7 +471,7 @@ e_source_list_peek_source_by_uid (ESourceList *list,
 	for (p = list->priv->groups; p != NULL; p = p->next) {
 		ESourceGroup *group = E_SOURCE_GROUP (p->data);
 		ESource *source;
-		
+
 		source = e_source_group_peek_source_by_uid (group, uid);
 		if (source)
 			return source;
@@ -499,7 +490,7 @@ e_source_list_peek_source_any (ESourceList *list)
 	for (p = list->priv->groups; p != NULL; p = p->next) {
 		ESourceGroup *group = E_SOURCE_GROUP (p->data);
 		GSList *sources;
-		
+
 		sources = e_source_group_peek_sources (group);
 		if (sources && sources->data)
 			return E_SOURCE (sources->data);
@@ -549,7 +540,7 @@ e_source_list_remove_group_by_uid (ESourceList *list,
 				    const char *uid)
 {
 	ESourceGroup *group;
-	
+
 	g_return_val_if_fail (E_IS_SOURCE_LIST (list), FALSE);
 	g_return_val_if_fail (uid != NULL, FALSE);
 
@@ -566,14 +557,14 @@ e_source_list_remove_source_by_uid (ESourceList *list,
 				     const char *uid)
 {
 	GSList *p;
-	
+
 	g_return_val_if_fail (E_IS_SOURCE_LIST (list), FALSE);
 	g_return_val_if_fail (uid != NULL, FALSE);
 
 	for (p = list->priv->groups; p != NULL; p = p->next) {
 		ESourceGroup *group = E_SOURCE_GROUP (p->data);
 		ESource *source;
-		
+
 		source = e_source_group_peek_source_by_uid (group, uid);
 		if (source)
 			return e_source_group_remove_source_by_uid (group, uid);
@@ -598,14 +589,111 @@ e_source_list_sync (ESourceList *list,
 		conf_list = g_slist_prepend (conf_list, e_source_group_to_xml (E_SOURCE_GROUP (p->data)));
 	conf_list = g_slist_reverse (conf_list);
 
-	retval = gconf_client_set_list (list->priv->gconf_client,
-					list->priv->gconf_path,
-					GCONF_VALUE_STRING,
-					conf_list,
-					error);
+	if (!e_source_list_is_gconf_updated (list))
+		retval = gconf_client_set_list (list->priv->gconf_client,
+						list->priv->gconf_path,
+						GCONF_VALUE_STRING,
+						conf_list,
+						error);
+	else
+		retval = TRUE;
 
 	g_slist_foreach (conf_list, (GFunc) g_free, NULL);
 	g_slist_free (conf_list);
 
 	return retval;
 }
+
+gboolean
+e_source_list_is_gconf_updated (ESourceList *list)
+{
+	char *source_group_xml = NULL;
+	char *gconf_xml = NULL;
+	char *group_uid = NULL;
+	GSList *conf_list = NULL, *temp = NULL, *p = NULL;
+	xmlDocPtr xmldoc;
+	ESourceGroup *group = NULL;
+	GSList *groups = NULL;
+	gboolean conf_to_list = TRUE, list_to_conf = TRUE;
+
+	g_return_val_if_fail (list != NULL, FALSE);
+
+	conf_list = gconf_client_get_list (list->priv->gconf_client,
+					   list->priv->gconf_path,
+					   GCONF_VALUE_STRING, NULL);
+
+	/* From conf to list */
+
+	for (temp = conf_list; temp != NULL; temp = temp->next) {
+		gconf_xml = (char *)temp->data;
+		xmldoc = xmlParseDoc ((const xmlChar *)gconf_xml);
+
+		group_uid = e_source_group_uid_from_xmldoc (xmldoc);
+		group = e_source_list_peek_group_by_uid (list, group_uid);
+		g_free (group_uid);
+		xmlFreeDoc (xmldoc);
+
+		if (group) {
+			source_group_xml = e_source_group_to_xml (group);
+			if (!strcmp (gconf_xml, source_group_xml)) {
+				g_free (source_group_xml);
+				continue;
+			}
+			else {
+				conf_to_list  = FALSE;
+				g_free (source_group_xml);
+				break;
+			}
+		} else {
+			conf_to_list = FALSE;
+			break;
+		}
+	}
+
+
+	/* If there is mismatch, free the conf_list and return FALSE */
+	if (!conf_to_list) {
+		for (p = conf_list; p != NULL ; p = p->next) {
+			gconf_xml = (char *) p->data;
+			g_free (gconf_xml);
+		}
+		g_slist_free (conf_list);
+		return FALSE;
+	}
+
+	groups = e_source_list_peek_groups (list);
+
+	/* From list to conf */
+
+	for (p = groups ; p != NULL; p = p->next) {
+		group = E_SOURCE_GROUP (p->data);
+		source_group_xml = e_source_group_to_xml (group);
+
+		for (temp = conf_list; temp != NULL; temp = temp->next) {
+			gconf_xml = (char *)temp->data;
+			if (strcmp (gconf_xml, source_group_xml))
+				continue;
+			else
+				break;
+		}
+		g_free (source_group_xml);
+
+		if (!temp) {
+			list_to_conf = FALSE;
+			break;
+		}
+
+	}
+
+	for (p = conf_list; p != NULL ; p = p->next) {
+		gconf_xml = (char *) p->data;
+		g_free (gconf_xml);
+	}
+	g_slist_free (conf_list);
+
+	/* If there is mismatch return FALSE */
+	if (!list_to_conf)
+		return FALSE;
+
+	return TRUE;
+}
